const Logger = console;
let Vue;
let GLOBAL_THERSHOLD;
let observer;
let count = 0;
const unit = 0.1;
let threshold = [];
while (count <= 1) {
    threshold.push(count);
    count = Number((count + unit).toFixed(2));
}
const OBSERVER_OPTIONS = {
    delay: 100,
    threshold: threshold,
    trackVisibility: true,
};
const elToMeta = new Map();
const useResetExposure = function (el) {
    if (el && elToMeta.has(el)) {
        const config = elToMeta.get(el);
        elToMeta.set(el, Object.assign(config, { active: false }));
    }
    else {
        for (let [key, config] of elToMeta.entries()) {
            if (config.active) {
                elToMeta.set(key, Object.assign(config, { active: false }));
            }
        }
    }
};
const createObserver = () => {
    if (window.IntersectionObserver && !observer) {
        observer = new window.IntersectionObserver((list, observer) => {
            for (let entry of list) {
                const { isIntersecting, target, intersectionRatio } = entry;
                if (isIntersecting) {
                    const config = elToMeta.get(target);
                    if (config &&
                        !config.active &&
                        intersectionRatio >= config.threshold) {
                        const { visibility, height, width } = window.getComputedStyle(target, null);
                        if (visibility === 'hidden' ||
                            parseInt(height) === 0 ||
                            parseInt(width) === 0) {
                            break;
                        }
                        typeof config.callback === 'function' && config.callback(target);
                        elToMeta.set(target, Object.assign(config, { active: true }));
                    }
                }
            }
        }, OBSERVER_OPTIONS);
    }
    if (!observer) {
        Logger.warn('current browser does not support IntersectionObserve API');
    }
};
const addElToObserve = (el, threshold, callback) => {
    if (!elToMeta.has(el)) {
        elToMeta.set(el, {
            active: false,
            callback,
            threshold,
        });
        observer && observer.observe(el);
    }
};
const mounted = (el, binding, vnode) => {
    let { value, arg } = binding;
    let threshold;
    if (typeof value !== 'function') {
        Logger.error('directive value is not function ');
        return;
    }
    threshold = Number(arg);
    if ((arg && typeof arg !== 'number') || !arg) {
        arg && Logger.error('element arguments  must be number type');
        threshold = GLOBAL_THERSHOLD || 1;
    }
    addElToObserve(el, threshold, value);
};
const beforeUnmount = (el) => {
    if (elToMeta.has(el) && observer) {
        elToMeta.delete(el);
        observer.unobserve(el);
    }
};
const installDirective = () => {
    Vue.directive('exposure', {
        mounted,
        beforeUnmount,
    });
};
const install = (_Vue, options) => {
    if (!Vue) {
        Vue = _Vue;
    }
    if (options && options.threshold) {
        GLOBAL_THERSHOLD = options.threshold;
    }
    createObserver();
    installDirective();
};
const Exposure = {
    install,
};

export default Exposure;
export { useResetExposure };
